import unittest

from nanoservice import Subscriber
from nanoservice import Publisher


class BaseTestCase(unittest.TestCase):

    def setUp(self):
        self.addr = 'inproc://test'
        self.client = Publisher(self.addr)
        self.service = Subscriber(self.addr)
        self.service.subscribe('upper', lambda line: line.upper())
        self.service.subscribe('lower', lambda line: line.lower())

    def tearDown(self):
        self.client.sock.close()
        self.service.sock.close()


class TestPubSub(BaseTestCase):

    def test_pub_sub(self):
        # Client side
        line = 'hello world'
        self.client.publish('upper', line)
        self.client.publish('lower', line.upper())

        # Server side
        uppercase = self.service.process()
        self.assertEqual(uppercase, line.upper())

        lowercase = self.service.process()
        self.assertEqual(lowercase, line.lower())


if __name__ == '__main__':
    unittest.main()



'''
The MIT License (MIT)

Copyright (c) 2014 Tony Walker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

'''

import uuid
import nanomsg
import logging

from .crypto import Authenticator
from .encoder import MsgPackEncoder


class Client(object):
    """ A requester client """

    # pylint: disable=too-many-arguments
    # pylint: disable=no-member
    def __init__(self, addr, encoder=None, socket=None,
                 auth=False, secret=None, digestmod=None):
        self.addr = addr
        self.encoder = encoder or MsgPackEncoder()
        self.sock = socket if socket else nanomsg.Socket(nanomsg.REQ)
        self.sock.connect(self.addr)
        self.authenticator = Authenticator(secret, digestmod) if auth else None

    @classmethod
    def build_payload(cls, method, args):
        """Add an uuid for each call"""
        ref = str(uuid.uuid4())
        return (method, args, ref)

    def encode(self, payload):
        """Encode the message via the provider encoder"""
        return self.encoder.encode(payload)

    def decode(self, msg):
        """Decode the message via the provider encoder"""
        return self.encoder.decode(msg)

    def send(self, payload):
        """ Send payload through the socket """
        payload = self.encode(payload)
        if self.authenticator:
            payload = self.authenticator.signed(payload)
        self.sock.send(payload)

    def receive(self):
        """ Receive response from socket """
        response = self.sock.recv()
        if self.authenticator:
            self.authenticator.auth(response)
            response = self.authenticator.unsigned(response)
        decoded = self.encoder.decode(response)
        return decoded

    # pylint: disable=logging-format-interpolation
    def call(self, method, *args):
        """ Call the remote service """
        payload = self.build_payload(method, args)
        logging.debug('* Client will send payload: {}'.format(payload))
        self.send(payload)

        res = self.receive()
        assert payload[2] == res['ref']
        return res['result'], res['error']



import unittest

from nanoservice import crypto
from nanoservice import error


class TestAuthenticator(unittest.TestCase):

    def setUp(self):
        self.authenticator = crypto.Authenticator('my secret')

    def tearDown(self):
        pass

    def test_good_signature(self):
        message = b'message'
        signed_message = self.authenticator.signed(message)
        self.authenticator.auth(signed_message)

    def test_bad_signature(self):
        message = b'message'
        signed = self.authenticator.signed(message)
        fake_sig = self.authenticator.digestmod('fake'.encode('utf-8'))
        fake_sig = fake_sig.hexdigest().encode('utf-8')
        signed = message + fake_sig
        with self.assertRaises(error.AuthenticatorInvalidSignature):
            self.authenticator.auth(signed)

    def test_unsigned(self):
        message = b'my super secret message'
        signed = self.authenticator.signed(message)
        print('signed is:', signed)
        unsigned = self.authenticator.unsigned(signed)
        self.assertEqual(message, unsigned)

if __name__ == '__main__':
    unittest.main()



""" Read configuration for a service from a json file """

import io
import re
import json

from .client import Client


class DotDict(dict):
    """ Access a dictionary like an object """

    def __getattr__(self, key):
        return self[key]

    def __setattr__(self, key, value):
        self[key] = value


def load(filepath=None, filecontent=None, clients=True, rename=True):
    """ Read the json file located at `filepath`

    If `filecontent` is specified, its content will be json decoded
    and loaded instead. The `clients` arg is a binary flag
    which specifies whether the endpoints present in config (`filecontent`),
    should be used to create `Client` objects.

    Usage:
        config.load(filepath=None, filecontent=None):
        Provide either a filepath or a json string
    """
    conf = DotDict()

    # Read json configuration
    assert filepath or filecontent
    if not filecontent:
        with io.FileIO(filepath) as handle:
            filecontent = handle.read().decode('utf-8')
    configs = json.loads(filecontent)

    # Update the conf items (Create clients if necessary)
    for key, value in configs.items():
        conf[key] = value
        if clients and isinstance(value, str) and \
                re.match('inproc:|ipc:|tcp:', value) and '.client' in key:
            conf[key] = Client(value)
        if rename:
            if key.endswith('.client'):
                new_key = key.replace('.client', '')
                conf[new_key] = conf.pop(key)
    return conf



import unittest
from multiprocessing import Process

from nanoservice import Service
from nanoservice import Client

from nanoservice import encoder


class BaseTestCase(unittest.TestCase):

    def setUp(self):
        self.addr = 'ipc:///tmp/test-service01.sock'

    def tearDown(self):
        self.client.sock.close()

    def start_service(self, addr):
        s = Service(addr)
        s.register('divide', lambda x, y: x/y)
        s.start()


class TestTCPProtocol(BaseTestCase):

    def make_req(self, *args):
        proc = Process(target=self.start_service, args=(self.addr,))
        proc.start()
        self.client = Client(self.addr)
        res, err = self.client.call('divide', *args)
        proc.terminate()
        return res, err

    def test_req_rep_w_success(self):
        res, err = self.make_req(6, 2)
        self.assertEqual(3, res)
        self.assertTrue(err is None)

    def test_req_rep_w_error(self):
        res, err = self.make_req(6, 0)
        self.assertTrue(res is None)
        self.assertTrue(err is not None)


class TestErrors(BaseTestCase):

    def make_req(self, *args):
        proc = Process(target=self.start_service, args=(self.addr,))
        proc.start()
        self.client = Client(self.addr)

        # Change encoder to force service to fail on encoding
        # since the service uses a MsgPack encoder
        self.client.encoder = encoder.JSONEncoder()

        # Build and send payload to service to trigger decoding
        payload = self.client.build_payload('divide', args)
        self.client.send(payload)

        # Change back to msg pack encoder to read the service response
        self.client.encoder = encoder.MsgPackEncoder()

        out = self.client.receive()
        proc.terminate()
        return out

    def test_decode_error(self):
        out = self.make_req(6, 2)
        # response shoud be empty since there was a decode error
        self.assertEqual('', out)


if __name__ == '__main__':
    unittest.main()



'''
The MIT License (MIT)

Copyright (c) 2014 Tony Walker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

'''

import sys
import signal
import nanomsg
import logging
import threading

from .crypto import Authenticator
from .encoder import MsgPackEncoder

from .error import DecodeError
from .error import RequestParseError
from .error import AuthenticateError
from .error import AuthenticatorInvalidSignature


class Service(object):
    """ A service which responds to requests """

    # pylint: disable=too-many-arguments
    # pylint: disable=no-member
    def __init__(self, addr, encoder=None, socket=None,
                 auth=False, secret=None, digestmod=None):
        self.addr = addr
        self.encoder = encoder or MsgPackEncoder()
        self.methods = {}
        self.descriptions = {}
        self.sock = socket if socket else nanomsg.Socket(nanomsg.REP)
        self.sock.bind(self.addr)
        self.authenticator = Authenticator(secret, digestmod) if auth else None

    def authenticate(self, payload):
        """ Authenticate payload then return unsigned payload """
        if not self.authenticator:
            return payload
        try:
            self.authenticator.auth(payload)
            return self.authenticator.unsigned(payload)
        except AuthenticatorInvalidSignature:
            raise
        except Exception as exception:
            raise AuthenticateError(str(exception))

    def decode(self, payload):
        """ Decode payload """
        try:
            return self.encoder.decode(payload)
        except Exception as exception:
            raise DecodeError(str(exception))

    def receive(self):
        """ Receive from socket, authenticate and decode payload """
        payload = self.sock.recv()
        payload = self.authenticate(payload)
        payload = self.decode(payload)
        return payload

    def send(self, response):
        """ Encode and sign (optional) the send through socket """
        response = self.encoder.encode(response)
        if self.authenticator:
            response = self.authenticator.signed(response)
        self.sock.send(response)

    def execute(self, method, args, ref):
        """ Execute the method with args """

        response = {'result': None, 'error': None, 'ref': ref}
        fun = self.methods.get(method)
        if not fun:
            response['error'] = 'Method `{}` not found'.format(method)
        else:
            try:
                response['result'] = fun(*args)
            except Exception as exception:
                logging.error(exception, exc_info=1)
                response['error'] = str(exception)
        return response

    def register(self, name, fun, description=None):
        """ Register function on this service """
        self.methods[name] = fun
        self.descriptions[name] = description

    @classmethod
    def parse(cls, payload):
        """ Parse client request """
        try:
            method, args, ref = payload
        except Exception as exception:
            raise RequestParseError(exception)
        else:
            return method, args, ref

    # pylint: disable=logging-format-interpolation
    def process(self):
        """ Receive data from socket and process request """

        response = None

        try:
            payload = self.receive()
            method, args, ref = self.parse(payload)
            response = self.execute(method, args, ref)

        except AuthenticateError as exception:
            logging.error(
                'Service Error in authenticate {}'.format(exception), exc_info=1)

        except AuthenticatorInvalidSignature as exception:
            logging.error(
                'Service Error authenticating {}'.format(exception), exc_info=1)

        except DecodeError as exception:
            logging.error(
                'Service Error authenticating {}'.format(exception), exc_info=1)

        except RequestParseError as exception:
            logging.error(
                'Service Error parsing {}'.format(exception), exc_info=1)

        else:
            logging.debug('Service Server received payload: {}'.format(payload))

        if response:
            self.send(response)
        else:
            self.send('')

    def start(self):
        """ Start and listen for calls """

        if threading.current_thread().name == 'MainThread':
            signal.signal(signal.SIGINT, self.stop)

        logging.info('Service started on {}'.format(self.addr))
        while True:
            self.process()

    def stop(self, dummy_signum=None, dummy_frame=None):
        """ Shutdown the service (this method is also a signal handler) """
        logging.info('Shutting down service ...')
        self.sock.close()
        sys.exit(0)



import unittest

from nanoservice import Service
from nanoservice import Client

from nanoservice import error
from nanoservice import crypto


class BaseTestCase(unittest.TestCase):

    def setUp(self):
        addr = 'inproc://test'
        self.client = Client(addr)
        self.service = Service(addr)
        self.service.register('divide', lambda x, y: x/y)
        self.service.register('echo', lambda x: x)

    def tearDown(self):
        self.client.sock.close()
        self.service.sock.close()


class TestService(BaseTestCase):

    def test_register_method(self):
        f = lambda x: x
        self.service.register(f, 'afun')

    def test_recv_method(self):
        sent = ['a', 'b', 'c']
        encoded = self.client.encoder.encode(sent)
        self.client.sock.send(encoded)
        got = self.service.receive()
        self.assertEqual(sent, got)

    def test_send_method(self):
        sent = ['a', 'b', 'c']
        self.test_recv_method()  # To send data to service
        self.service.send(sent)
        got = self.client.encoder.decode(self.client.sock.recv())
        self.assertEqual(sent, got)

    def test_execute_method_w_success(self):
        res = self.service.execute('divide', (6, 2), None)
        expected = {'result': 3, 'error': None, 'ref': None}
        self.assertEqual(res, expected)

    def test_execute_method_w_error(self):
        res = self.service.execute('divide', (1, 0), None)
        self.assertIsNotNone(res['error'])

    def test_encoder(self):
        data = {'name': 'Joe Doe'}
        encoded = self.service.encoder.encode(data)
        decoded = self.service.encoder.decode(encoded)
        self.assertEqual(data, decoded)

    def test_service_authenticate_simple(self):
        payload = 'no authentication set up. payload will be unchanged'
        got = self.service.authenticate(payload)
        self.assertEqual(got, payload)

    # Test error throwing

    def test_payload_decode_error(self):
        self.client.sock.send('"abc')
        self.assertRaises(error.DecodeError, self.service.receive)

    def test_payload_parse_error(self):
        for payload in [[1, 2], '', None]:
            self.assertRaises(
                error.RequestParseError, self.service.parse, payload)

    def test_payload_authenticate_error(self):
        payload = 'abc'
        auth = crypto.Authenticator('my secret')
        auth.unsigned = None  # Overwrite method to force error

        self.service.authenticator = auth
        self.client.authenticator = auth

        payload = self.client.encode(payload)
        payload = auth.signed(payload)
        self.client.sock.send(payload)
        self.assertRaises(
            error.AuthenticateError, self.service.receive)

    def test_payload_authenticator_invalid_signature(self):
        payload = 'abc'
        auth = crypto.Authenticator('my secret')
        self.service.authenticator = auth
        self.client.authenticator = auth

        payload = self.client.encode(payload)
        payload = auth.signed(payload)
        self.client.sock.send(b'123' + payload)
        self.assertRaises(
            error.AuthenticatorInvalidSignature, self.service.receive)


if __name__ == '__main__':
    unittest.main()



"""NanoService Exceptions"""


class NanoServiceError(Exception):
    """Base exception for Nanoservice"""
    pass

class ServiceError(NanoServiceError):
    """Service Generic Exception"""
    pass


class ClientError(NanoServiceError):
    """Client Generic Exception"""
    pass


class ConfigError(NanoServiceError):
    """Config Generic Exception"""
    pass


class AuthenticatorInvalidSignature(NanoServiceError):
    """ Message could not be authenticated """
    pass


class RequestParseError(NanoServiceError):
    """ Message from client could not be parsed """
    pass


class PublisherError(NanoServiceError):
    """Publisher Generic Exception"""
    pass

class SubscriberError(NameError):
    """Subscriber Generic Exception"""
    pass


class DecodeError(ServiceError):
    """ Service cannot decode request """
    pass


class AuthenticateError(ServiceError):
    """ Service cannot authenticate request """
    pass



import unittest

from nanoservice import Service
from nanoservice import Client


class BaseTestCase(unittest.TestCase):

    def setUp(self):
        addr = 'inproc://test'
        self.client = Client(addr)
        self.service = Service(addr)
        self.service.register('divide', lambda x, y: x/y)
        self.service.register('echo', lambda x: x)

    def tearDown(self):
        self.client.sock.close()
        self.service.sock.close()


class TestClient(BaseTestCase):

    def test_build_payload(self):
        payload = self.client.build_payload('echo', 'My Name')
        method, args, ref = payload
        self.assertTrue(method == 'echo')
        self.assertTrue(len(payload) == 3)

    def test_encoder(self):
        data = {'name': 'Joe Doe'}
        encoded = self.client.encode(data)
        decoded = self.client.decode(encoded)
        self.assertEqual(data, decoded)

    def test_call_wo_receive(self):
        # Client side ops
        method, args = 'echo', 'hello world'
        payload = self.client.build_payload(method, args)
        self.client.sock.send(self.client.encode(payload))
        # Service side ops
        method, args, ref = self.service.receive()
        self.assertEqual(method, 'echo')
        self.assertEqual(args, 'hello world')
        self.assertEqual(ref, payload[2])

    def test_basic_socket_operation(self):
        msg = 'abc'
        self.client.sock.send(msg)
        res = self.service.sock.recv().decode('utf-8')
        self.assertEqual(msg, res)


if __name__ == '__main__':
    unittest.main()



"""Main module for nanoservice"""

from nanoservice.client import Client
from nanoservice.service import Service
from nanoservice.pubsub import Subscriber, Publisher
from nanoservice.crypto import Authenticator
from nanoservice.error import (
    NanoServiceError,
    ServiceError,
    ClientError,
    ConfigError,
    AuthenticatorInvalidSignature,
    RequestParseError,
    PublisherError,
    SubscriberError,
    DecodeError,
    AuthenticateError
)

__all__ = [
    'Client', 'Service', 'Subscriber', 'Publisher', 'Authenticator',
    'NanoServiceError', 'ServiceError', 'ClientError', 'ConfigError',
    'AuthenticatorInvalidSignature', 'RequestParseError',
    'PublisherError', 'SubscriberError', 'DecodeError', 'AuthenticateError'

]



"""Basic message signature for NanoService"""

import hmac
import hashlib

from .error import AuthenticatorInvalidSignature


class Authenticator(object):
    """ This object is used to authenticate messages """

    def __init__(self, secret, digestmod=None):
        assert secret
        self.secret = secret.encode('utf-8')
        self.digestmod = digestmod or hashlib.sha256
        self.sig_size = self.digestmod().digest_size * 2
        self._hmac = hmac.new(self.secret, digestmod=self.digestmod)

    def sign(self, encoded):
        """ Return authentication signature of encoded bytes """
        signature = self._hmac.copy()
        signature.update(encoded)
        return signature.hexdigest().encode('utf-8')

    def signed(self, encoded):
        """ Sign encoded bytes and append signature """
        signature = self.sign(encoded)
        return encoded + signature

    def unsigned(self, encoded):
        """ Remove signature and return just the message """
        message, _ = self.split(encoded)
        return message

    def split(self, encoded):
        """ Split into signature and message """
        maxlen = len(encoded) - self.sig_size
        message = encoded[:maxlen]
        signature = encoded[-self.sig_size:]
        return message, signature

    def auth(self, encoded):
        """ Validate integrity of encoded bytes """
        message, signature = self.split(encoded)
        computed = self.sign(message)
        if not hmac.compare_digest(signature, computed):
            raise AuthenticatorInvalidSignature



import json
import unittest

from nanoservice import config


class TestConfig(unittest.TestCase):

    def setUp(self):
        self.addr = 'ipc:///tmp/test-service.sock'
        self.expected = {
            "db.name": "test",
            "service.endpoint": self.addr
        }

    def tearDown(self):
        pass

    def test_config_load(self):
        C = config.load("test/testdata/service.json", clients=False)
        self.assertTrue(C.keys(), self.expected.keys())
        self.assertTrue(C.values(), self.expected.values())

    def test_config_load_with_content(self):
        filec = json.dumps(self.expected)
        C = config.load(filecontent=filec, clients=False)
        self.assertTrue(C.keys(), self.expected.keys())
        self.assertTrue(C.values(), self.expected.values())

    def test_dot_dict_get_and_set(self):
        d = config.DotDict()
        d['name'] = 'John Doe'
        self.assertEqual(getattr(d, 'name'), 'John Doe')
        d.age = '20'
        self.assertEqual(getattr(d, 'age'), '20')


if __name__ == '__main__':
    unittest.main()



"""Nanoservice PUBSUB"""

import nanomsg
import logging

from .service import Service
from .client import Client

from .error import SubscriberError
from .error import PublisherError

from .error import DecodeError
from .error import RequestParseError
from .error import AuthenticateError
from .error import AuthenticatorInvalidSignature


class Subscriber(Service):
    """ Subscriber """

    # pylint: disable=too-many-arguments
    # pylint: disable=no-member
    def __init__(self, addr, encoder=None,
                 auth=False, secret=None, digestmod=None):
        socket = nanomsg.Socket(nanomsg.SUB)
        super(Subscriber, self).__init__(
            addr, encoder, socket, auth, secret, digestmod)

    def get_fun_and_data(self, subscription):
        """ Fetch the function registered for a certain subscription """

        for name in self.methods:
            tag = bytes(name.encode('utf-8'))
            if subscription.startswith(tag):
                fun = self.methods.get(name)
                data = self.encoder.decode(subscription[len(tag):])
                return fun, data
        return None, None

    def register(self, name, fun, description=None):
        raise SubscriberError('Operation not allowed on this type of service')

    # pylint: disable=no-member
    def subscribe(self, tag, fun):
        """ Subscribe and register a function """
        super(Subscriber, self).register(tag, fun)
        self.sock.set_string_option(nanomsg.SUB, nanomsg.SUB_SUBSCRIBE, tag)

    def receive(self):
        """ Receive request from client """
        payload = self.sock.recv()
        if self.authenticator:
            self.authenticator.auth(payload)
            payload = self.authenticator.unsigned(payload)
        return payload

    # pylint: disable=logging-format-interpolation
    # pylint: disable=duplicate-code
    def process(self):

        try:
            subscription = self.receive()

        except AuthenticateError as exception:
            logging.error('PubSub Error in authenticate {}'.format(exception), exc_info=1)

        except AuthenticatorInvalidSignature as exception:
            logging.error('PubSub Error authenticating {}'.format(exception), exc_info=1)

        except DecodeError as exception:
            logging.error('PubSub Error authenticating {}'.format(exception), exc_info=1)

        except RequestParseError as exception:
            logging.error('PubSub Error parsing {}'.format(exception), exc_info=1)

        else:
            logging.debug('PubSub Server received payload: {}'.format(subscription))

        fun, data = self.get_fun_and_data(subscription)

        result = None
        try:
            result = fun(data)
        except Exception as exception:
            logging.error(exception, exc_info=1)

        # Return result to check successful execution of `fun` when testing
        return result


class Publisher(Client):
    """ Publisher """

    #pylint: disable=too-many-arguments
    # pylint: disable=no-member
    def __init__(self, addr, encoder=None,
                 auth=False, secret=None, digestmod=None):
        socket = nanomsg.Socket(nanomsg.PUB)
        super(Publisher, self).__init__(
            addr, encoder, socket, auth, secret, digestmod)

    def call(self, method, *args):
        raise PublisherError('Operation not allowed')

    def send(self, payload):
        """ Send payload through the socket """
        if self.authenticator:
            payload = self.authenticator.signed(payload)
        self.sock.send(payload)

    def build_payload(self, tag, data):
        payload = bytes(tag.encode('utf-8')) + self.encode(data)
        return payload

    def publish(self, tag, data):
        """Publish a message"""
        payload = self.build_payload(tag, data)
        self.send(payload)



from multiprocessing import Process

from nanoservice import Service
from nanoservice import Client
from nanoservice import encoder


def check(res, expected):
    assert res == expected


def start_service(addr, encoder, auth=False, secret=None):
    """ Start a service with options """
    s = Service(addr, encoder=encoder, auth=auth, secret=secret)
    s.register('none', lambda: None)
    s.register('divide', lambda x, y: x/y)
    s.register('upper', lambda dct: {k: v.upper() for k, v in dct.items()})
    s.start()


# ------------------


TESTS = [
    (('divide', [10, 2]), 5.0),
    (('none', []), None),
    (('upper', [{'a': 'a'}]), {'a': 'A'})
]


def test_encoding():
    """ Test encoding with defferent options """
    addr = 'ipc:///tmp/test-service01.sock'

    auth_schemes = [(False, None), (True, 'my secret')]
    encoders = [encoder.JSONEncoder(), encoder.MsgPackEncoder()]

    for test, expected in TESTS:
        for enc in encoders:
            for scheme in auth_schemes:

                method, args = test
                auth, secret = scheme

                proc = Process(
                    target=start_service,
                    args=(addr, enc, auth, secret))
                proc.start()

                client = Client(addr, encoder=enc, auth=auth, secret=secret)
                res, err = client.call(method, *args)
                client.sock.close()
                proc.terminate()
                yield check, res, expected
                # self.assertEqual(expected, res)



"""Nanoservice encoders"""

import json
import logging
import msgpack


class Encoder(object):
    """Base encoder class"""

    # pylint: disable=unused-argument
    @classmethod
    def encode(cls, data):
        """Base encode function"""
        logging.error('Encoding fun not implemented')

    # pylint: disable=unused-argument
    @classmethod
    def decode(cls, data):
        """Base decode function"""
        logging.error('Decoding fun not implemented')


class JSONEncoder(Encoder):
    """Json encoder for nanoservice message"""
    def __init__(self):
        super(JSONEncoder, self).__init__()

    def encode(self, data):
        return json.dumps(data).encode('utf-8')

    def decode(self, data):
        return json.loads(data.decode('utf-8'))


class MsgPackEncoder(Encoder):
    """MsgPack encoder for nanoservice message"""
    def __init__(self):
        super(MsgPackEncoder, self).__init__()

    def encode(self, data):
        return msgpack.packb(data)

    def decode(self, data):
        return msgpack.unpackb(data, encoding='utf-8')



"""Nanoservice installation script

https://github.com/walkr/nanoservice
"""

#!/usr/bin/env python

import sys

from setuptools import setup


def read_long_description(readme_file):
    """ Read package long description from README file """
    try:
        import pypandoc
    except (ImportError, OSError) as exception:
        print('No pypandoc or pandoc: %s' % (exception,))
        if sys.version_info.major == 3:
            handle = open(readme_file, encoding='utf-8')
        else:
            handle = open(readme_file)
        long_description = handle.read()
        handle.close()
        return long_description
    else:
        return pypandoc.convert(readme_file, 'rst')

setup(
    name='nanoservice',
    version='0.5.3',
    packages=['nanoservice'],
    author='Tony Walker',
    author_email='walkr.walkr@gmail.com',
    url='https://github.com/walkr/nanoservice',
    license='MIT',
    description='nanoservice is a small Python library for '
                'writing lightweight networked services using nanomsg',
    long_description=read_long_description('README.md'),
    install_requires=[
        'msgpack-python',
        'nanomsg',
        'nose',
    ],
    dependency_links=[
        'git+https://github.com/tonysimpson/nanomsg-python.git@master#egg=nanomsg',
    ],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
)
